import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
//import java.awt.event.WindowEvent;
//import java.awt.event.WindowListener;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.io.File;
import java.util.ArrayList;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 * Dialogfenster fr die Registermaschine
 * 
 * @author Bernd Gramlich 
 * @version 2015-01-19
 */
public class Dialogfenster implements ActionListener, FocusListener, MaschinenView, MaschinenBeobachter
{
    /*
     * Das MaschinenModel, das ich beobachte.
     */
    private MaschinenModel model;

    /*
     * Der MaschinenController, der mich steuert.
     */
    private MaschinenController controller;

    /*
     * Die Komponenten des Dialogfensters.
     */
    private JMenuItem neuMenueEintrag;
    private JMenuItem ladenMenueEintrag;
    private JMenuItem speichernMenueEintrag;
    private JMenuItem speichernUnterMenueEintrag;
    private JMenuItem endeMenueEintrag;
    private JMenu programmMenue;
    private JMenuItem laufenMenueEintrag;
    private JMenuItem schrittweiseMenueEintrag;
    private JMenuItem unterbrechenMenueEintrag;
    private JMenuItem fortsetzenMenueEintrag;
    private JMenuItem abbrechenMenueEintrag;
    private JMenu laufenMenue;
    private JCheckBoxMenuItem nurPositivMenueEintrag;
    private JRadioButtonMenuItem binaerMenueEintrag;
    private JRadioButtonMenuItem oktalMenueEintrag;
    private JRadioButtonMenuItem dezimalMenueEintrag;
    private JRadioButtonMenuItem hexadezimalMenueEintrag;
    private ButtonGroup zahlenGruppe;
    private JMenu zahlenMenue;
    private JMenuItem langsamerMenueEintrag;
    private JMenuItem taktzahlMenueEintrag;
    private JMenuItem schnellerMenueEintrag;
    private JMenu geschwindigkeitMenue;
    private JMenuBar menueLeiste;
    private GridBagConstraints gitter;
    private JLabel akkumulatorUeberschrift;
    private JTextField akkumulatorEingabe;
    private JLabel befehlszaehlerUeberschrift;
    private JTextField befehlszaehlerEingabe;
    private JPanel zentralRaum;
    private JLabel programmUeberschrift;
    private JLabel[] befehlsNummer;
    private JComboBox[] befehlsAuswahl;
    private JTextField[] argumentEingabe;
    private JPanel[] befehlsRaum;
    private JPanel programmRaum;
    private JButton programmZurueckKnopf;
    private JButton programmVorKnopf;
    private JPanel programmKnopfRaum;
    private JLabel speicherUeberschrift;
    private JLabel[] zellenNummer;
    private JTextField[] zellenEingabe;
    private JPanel[] zellenRaum;
    private JPanel speicherRaum;
    private JButton speicherZurueckKnopf;
    private JButton speicherVorKnopf;
    private JPanel speicherKnopfRaum;
    private JLabel nachrichtenAusgabe;
    private JPanel fensterRaum;
    private JFrame fenster;

    /*
     * Die Anzahlen der sichtbaren Befehle, Zellen und Zeilen.
     */
    private final int BEFEHLSZAHL = 32;
    private final int ZELLENZAHL  = 64;
    private final int ZEILENZAHL  = 16;

    /*
     * Der Platzhalter, der die Gre von Beschriftungen festlegt.
     */
    private final String PLATZHALTER = "                         ";

    /*
     * Die Spaltenzahl, die die Gre von Textfeldern festlegt.
     */
    private final int SPALTENZAHL = 6;

    /*
     * Der Auswahldialog zum Laden und Speichern eines Programms.
     */
    private JFileChooser dateiAuswahl;

    /*
     * Die sichtbare Seitennummer des Programms.
     */
    private int programmSeite = 0;

    /*
     * Die sichtbare Seitennummer des Speichers.
     */
    private int speicherSeite = 0;

    /*
     * Das Programm als Liste von Programmzeilen.
     */
    private ArrayList<Befehl> programmZeile;

    /*
     * Die Argumente der einzelnen Programmzeilen.
     */
    private ArrayList<Integer> argument;

    /*
     * Die Inhalte der einzelnen Speicherzellen.
     */
    private ArrayList<Integer> zellenInhalt;
    
    /*
     * Der aktuelle Befehl (meistens gleich dem Befehlszaehler).
     */
    private int aktuelleBefehlsNummer;

    /*
     * Die aktuelle Speicherzelle.
     */
    private int aktuelleZellenNummer;

    /*
     * Hilfsfarben.
     */
    private Color vordergrund        = Color.black;
    private Color hintergrundOben    = Color.white;
    private Color hintergrundUnten   = Color.white;
    private Color hintergrundAuswahl = Color.white;

    /**
     * Konstruktor.
     */
    public Dialogfenster (MaschinenModel m, MaschinenController c)
    {
        model = m;
        controller = c;

        neuMenueEintrag            = new JMenuItem ("Neu");
        ladenMenueEintrag          = new JMenuItem ("Laden");
        speichernMenueEintrag      = new JMenuItem ("Speichern");
        speichernUnterMenueEintrag = new JMenuItem ("Speichern unter");
        endeMenueEintrag           = new JMenuItem ("Ende");

        neuMenueEintrag.addActionListener (this);
        endeMenueEintrag.addActionListener (this);

        programmMenue = new JMenu ("Programm");
        programmMenue.add (neuMenueEintrag);
        programmMenue.add (ladenMenueEintrag);
        programmMenue.add (speichernMenueEintrag);
        programmMenue.add (speichernUnterMenueEintrag);
        programmMenue.add (endeMenueEintrag);

        laufenMenueEintrag       = new JMenuItem ("Laufen");
        schrittweiseMenueEintrag = new JMenuItem ("Schrittweise");
        unterbrechenMenueEintrag = new JMenuItem ("Unterbrechen");
        fortsetzenMenueEintrag   = new JMenuItem ("Fortsetzen");
        abbrechenMenueEintrag    = new JMenuItem ("Abbrechen");

        laufenMenueEintrag.addActionListener (this);
        schrittweiseMenueEintrag.addActionListener (this);
        unterbrechenMenueEintrag.addActionListener (this);
        fortsetzenMenueEintrag.addActionListener (this);
        abbrechenMenueEintrag.addActionListener (this);

        laufenMenue = new JMenu ("Laufen");
        laufenMenue.add (laufenMenueEintrag);
        laufenMenue.add (schrittweiseMenueEintrag);
        laufenMenue.add (unterbrechenMenueEintrag);
        laufenMenue.add (fortsetzenMenueEintrag);
        laufenMenue.add (abbrechenMenueEintrag);

        nurPositivMenueEintrag  = new JCheckBoxMenuItem ("nur positive Zahlen und Null", false);
        binaerMenueEintrag      = new JRadioButtonMenuItem ("binr", false);
        oktalMenueEintrag       = new JRadioButtonMenuItem ("oktal", false);
        dezimalMenueEintrag     = new JRadioButtonMenuItem ("dezimal", true);
        hexadezimalMenueEintrag = new JRadioButtonMenuItem ("hexadezimal", false);
        
        nurPositivMenueEintrag.setEnabled (false);

        binaerMenueEintrag.addActionListener (this);
        oktalMenueEintrag.addActionListener (this);
        dezimalMenueEintrag.addActionListener (this);
        hexadezimalMenueEintrag.addActionListener (this);

        zahlenGruppe = new ButtonGroup ();
        zahlenGruppe.add (binaerMenueEintrag);
        zahlenGruppe.add (oktalMenueEintrag);
        zahlenGruppe.add (dezimalMenueEintrag);
        zahlenGruppe.add (hexadezimalMenueEintrag);

        zahlenMenue = new JMenu ("Zahlen");
        zahlenMenue.add (nurPositivMenueEintrag);
        zahlenMenue.addSeparator ();
        zahlenMenue.add (binaerMenueEintrag);
        zahlenMenue.add (oktalMenueEintrag);
        zahlenMenue.add (dezimalMenueEintrag);
        zahlenMenue.add (hexadezimalMenueEintrag);

        langsamerMenueEintrag = new JMenuItem ("langsamer");
        taktzahlMenueEintrag = new JMenuItem ("1000");
        schnellerMenueEintrag = new JMenuItem ("schneller");

        taktzahlMenueEintrag.setEnabled (false);

        geschwindigkeitMenue = new JMenu ("Geschwindigkeit");
        geschwindigkeitMenue.add (langsamerMenueEintrag);
        geschwindigkeitMenue.add (taktzahlMenueEintrag);
        geschwindigkeitMenue.add (schnellerMenueEintrag);

        menueLeiste = new JMenuBar ();
        menueLeiste.add (programmMenue);
        menueLeiste.add (laufenMenue);
        menueLeiste.add (zahlenMenue);
        menueLeiste.add (geschwindigkeitMenue);

        gitter = new GridBagConstraints ();

        akkumulatorUeberschrift    = new JLabel ("Akkumulator");
        akkumulatorEingabe         = new JTextField ();
        befehlszaehlerUeberschrift = new JLabel ("Befehlszhler");
        befehlszaehlerEingabe      = new JTextField ();

        akkumulatorUeberschrift.setHorizontalAlignment (JTextField.TRAILING);
        akkumulatorEingabe.setColumns (SPALTENZAHL);
        akkumulatorEingabe.setEditable (false);
        befehlszaehlerUeberschrift.setHorizontalAlignment (JTextField.TRAILING);
        befehlszaehlerEingabe.setColumns (SPALTENZAHL);
        befehlszaehlerEingabe.setEditable (false);

        zentralRaum= new JPanel (new GridBagLayout ());

        gitter.gridx = 0;
        gitter.gridy = 0;
        zentralRaum.add (akkumulatorUeberschrift, gitter);

        gitter.gridx = 1;
        gitter.gridy = 0;
        zentralRaum.add (akkumulatorEingabe, gitter);

        gitter.gridx = 0;
        gitter.gridy = 1;
        zentralRaum.add (befehlszaehlerUeberschrift, gitter);

        gitter.gridx = 1;
        gitter.gridy = 1;
        zentralRaum.add (befehlszaehlerEingabe, gitter);

        programmUeberschrift = new JLabel ("Programm");

        befehlsNummer   = new JLabel[BEFEHLSZAHL];
        befehlsAuswahl  = new JComboBox[BEFEHLSZAHL];
        argumentEingabe = new JTextField[BEFEHLSZAHL];
        befehlsRaum     = new JPanel[BEFEHLSZAHL];

        programmRaum = new JPanel (new GridBagLayout ());

        for (int i = 0; i < BEFEHLSZAHL; i++)
        {
            befehlsNummer[i]   = new JLabel (PLATZHALTER);
            befehlsAuswahl[i]  = new JComboBox (Befehl.values ());
            argumentEingabe[i] = new JTextField ();

            befehlsNummer[i].setHorizontalAlignment (JTextField.TRAILING);
            argumentEingabe[i].setColumns (SPALTENZAHL);
            argumentEingabe[i].setEditable (((Befehl) befehlsAuswahl[i].getSelectedItem ()).brauchtArgument () != 0);

            befehlsAuswahl[i].addActionListener (this);
            argumentEingabe[i].addFocusListener (this);

            befehlsRaum [i] = new JPanel (new FlowLayout ());
            befehlsRaum [i].add (befehlsNummer[i]);
            befehlsRaum [i].add (befehlsAuswahl[i]);
            befehlsRaum [i].add (argumentEingabe[i]);

            gitter.gridx = i / ZEILENZAHL;
            gitter.gridy = i % ZEILENZAHL;
            programmRaum.add (befehlsRaum [i], gitter);
        }

        programmZurueckKnopf = new JButton (" < ");
        programmVorKnopf     = new JButton (" > ");

        programmZurueckKnopf.addActionListener (this);
        programmVorKnopf.addActionListener (this);

        programmKnopfRaum = new JPanel (new FlowLayout ());
        programmKnopfRaum.add (programmZurueckKnopf);
        programmKnopfRaum.add (programmVorKnopf);

        speicherUeberschrift = new JLabel ("Speicher");

        zellenNummer  = new JLabel[ZELLENZAHL];
        zellenEingabe = new JTextField[ZELLENZAHL];
        zellenRaum    = new JPanel[ZELLENZAHL];

        speicherRaum = new JPanel (new GridBagLayout ());

        for (int i = 0; i < ZELLENZAHL; i++)
        {
            zellenNummer[i]  = new JLabel (PLATZHALTER);
            zellenEingabe[i] = new JTextField ();

            zellenNummer[i].setHorizontalAlignment (JTextField.TRAILING);
            zellenEingabe[i].setColumns (SPALTENZAHL);

            zellenEingabe[i].addFocusListener (this);

            zellenRaum [i] = new JPanel (new FlowLayout ());
            zellenRaum [i].add (zellenNummer[i]);
            zellenRaum [i].add (zellenEingabe[i]);

            gitter.gridx = i / ZEILENZAHL;
            gitter.gridy = i % ZEILENZAHL;
            speicherRaum.add (zellenRaum [i], gitter);
        }

        int befehlsBreite = (int) befehlsRaum[0].getPreferredSize ().getWidth ();
        int zellenBreite  = (int) zellenRaum[0].getPreferredSize ().getWidth ();
        int hoehe = (int) Math.max (befehlsRaum[0].getPreferredSize ().getHeight (), zellenRaum[0].getPreferredSize ().getHeight ());

        for (int i = 0; i < BEFEHLSZAHL; i++)
        {
            befehlsRaum[i].setPreferredSize (new Dimension (befehlsBreite, hoehe));
        }

        for (int i = 0; i < ZELLENZAHL; i++)
        {
            zellenRaum[i].setPreferredSize (new Dimension (zellenBreite, hoehe));
        }

        speicherZurueckKnopf = new JButton (" < ");
        speicherVorKnopf     = new JButton (" > ");

        speicherZurueckKnopf.addActionListener (this);
        speicherVorKnopf.addActionListener (this);

        speicherKnopfRaum = new JPanel (new FlowLayout ());
        speicherKnopfRaum.add (speicherZurueckKnopf);
        speicherKnopfRaum.add (speicherVorKnopf);

        nachrichtenAusgabe = new JLabel (PLATZHALTER);

        fensterRaum = new JPanel (new GridBagLayout ());

        gitter.gridx = 0;
        gitter.gridy = 0;
        gitter.gridwidth = 2;
        fensterRaum.add (zentralRaum, gitter);
        gitter.gridwidth = 1;

        gitter.gridx = 0;
        gitter.gridy = 1;
        fensterRaum.add (programmUeberschrift, gitter);

        gitter.gridx = 0;
        gitter.gridy = 2;
        fensterRaum.add (programmRaum, gitter);

        gitter.gridx = 0;
        gitter.gridy = 3;
        fensterRaum.add (programmKnopfRaum, gitter);

        gitter.gridx = 1;
        gitter.gridy = 1;
        fensterRaum.add (speicherUeberschrift, gitter);

        gitter.gridx = 1;
        gitter.gridy = 2;
        fensterRaum.add (speicherRaum, gitter);

        gitter.gridx = 1;
        gitter.gridy = 3;
        fensterRaum.add (speicherKnopfRaum, gitter);

        gitter.gridx = 0;
        gitter.gridy = 4;
        gitter.gridwidth = 2;
        fensterRaum.add (nachrichtenAusgabe, gitter);
        gitter.gridwidth = 1;

        fenster = new JFrame ("Registermaschine");
        fenster.setDefaultCloseOperation (JFrame.EXIT_ON_CLOSE);
        fenster.setJMenuBar (menueLeiste);
        fenster.getContentPane ().add (fensterRaum);
        fenster.pack ();
        fenster.setLocationRelativeTo (null);
        fenster.setVisible (true);

        /*
         * Beginne mit leerem Programm und leerem Speicherinhalt.
         */
        programmZeile = new ArrayList<Befehl> ();
        argument      = new ArrayList<Integer> ();
        zellenInhalt  = new ArrayList<Integer> ();

        beschrifteProgrammzeilen ();
        beschrifteSpeicherzellen ();

        befehlsAuswahl[0].requestFocus ();

        /*
         * Beobachte das Model.
         */
        model.speichereAlsBeobachter (this);
    }

    /**
     * Reagiere auf Benutzeraktionen.
     */
    public void actionPerformed (ActionEvent ereignis)
    {
        nachrichtenAusgabe.setText (PLATZHALTER);
        
        if (ereignis.getSource () == neuMenueEintrag)
        {
            controller.neu ();
        }
        else if (ereignis.getSource () == endeMenueEintrag)
        {
            controller.ende ();
        }
        else if (ereignis.getSource () == laufenMenueEintrag)
        {
            controller.laufe ();
        }
        else if (ereignis.getSource () == schrittweiseMenueEintrag)
        {
            controller.laufeSchrittweise ();
        }
        else if (ereignis.getSource () == unterbrechenMenueEintrag)
        {
            controller.unterbrich ();
        }
        else if (ereignis.getSource () == fortsetzenMenueEintrag)
        {
            controller.setzeFort ();
        }
        else if (ereignis.getSource () == abbrechenMenueEintrag)
        {
            controller.brichAb ();
        }
        else if (ereignis.getSource () == programmZurueckKnopf)
        {
            programmSeite--;
            beschrifteProgrammzeilen ();
        }
        else if (ereignis.getSource () == programmVorKnopf)
        {
            programmSeite++;
            beschrifteProgrammzeilen ();
        }
        else if (ereignis.getSource () == speicherZurueckKnopf)
        {
            speicherSeite--;
            beschrifteSpeicherzellen ();
        }
        else if (ereignis.getSource () == speicherVorKnopf)
        {
            speicherSeite++;
            beschrifteSpeicherzellen ();
        }
        else if (ereignis.getSource () == binaerMenueEintrag || ereignis.getSource () == oktalMenueEintrag || ereignis.getSource () == dezimalMenueEintrag || ereignis.getSource () == hexadezimalMenueEintrag)
        {
            beschrifteProgrammzeilen ();
            beschrifteSpeicherzellen ();
        }
        else
        {
            for (int i = 0; i < BEFEHLSZAHL; i++)
            {
                if (ereignis.getSource () == befehlsAuswahl[i])
                {
                    if (((Befehl) befehlsAuswahl[i].getSelectedItem ()).brauchtArgument () == 0)
                    {
                        argumentEingabe[i].setText ("");
                        argumentEingabe[i].setEditable (false);
                    }
                    else
                    {  
                        argumentEingabe[i].setEditable (true);
                        argumentEingabe[i].requestFocus ();
                    }

                    controller.setzeBefehl (programmSeite * BEFEHLSZAHL + i, (Befehl) befehlsAuswahl[i].getSelectedItem ());
                }
            }
        }
    }

    /**
     * Reagiere nicht auf erhaltenen Fokus.
     */
    public void focusGained (FocusEvent ereignis)
    {
    }

    /**
     * Reagiere auf verlorenen Fokus.
     */
    public void focusLost (FocusEvent ereignis)
    {
        for (int i = 0; i < BEFEHLSZAHL; i++)
        {
            if (ereignis.getSource () == argumentEingabe[i])
            {
                int zahl = 0;

                try
                {
                    if (binaerMenueEintrag.isSelected ())
                    {
                        zahl = Integer.parseInt (argumentEingabe[i].getText (), 2);
                    }
                    else if (oktalMenueEintrag.isSelected ())
                    {
                        zahl = Integer.parseInt (argumentEingabe[i].getText (), 8);
                    }
                    else if (dezimalMenueEintrag.isSelected ())
                    {
                        zahl = Integer.parseInt (argumentEingabe[i].getText ());
                    }
                    else if (hexadezimalMenueEintrag.isSelected ())
                    {
                        zahl = Integer.parseInt (argumentEingabe[i].getText (), 16);
                    }

                    if ((nurPositivMenueEintrag.isSelected () || ((Befehl) befehlsAuswahl[i].getSelectedItem ()).brauchtArgument () == 1) && zahl < 0)
                    {
                        nachrichtenAusgabe.setText ("Negative Zahlen sind nicht erlaubt!");
                        argumentEingabe[i].requestFocus ();
                    }
                    else
                    {
                        controller.setzeArgument (programmSeite * BEFEHLSZAHL + i, zahl);
                    }
                }
                catch (Exception fehler)
                {
                    nachrichtenAusgabe.setText ("Falsches Zahlenformat!");
                    argumentEingabe[i].requestFocus ();
                }
            }
        }

        for (int i = 0; i < ZELLENZAHL; i++)
        {
            if (ereignis.getSource () == zellenEingabe[i])
            {
                int zahl = 0;

                try
                {
                    if (binaerMenueEintrag.isSelected ())
                    {
                        zahl = Integer.parseInt (zellenEingabe[i].getText (), 2);
                    }
                    else if (oktalMenueEintrag.isSelected ())
                    {
                        zahl = Integer.parseInt (zellenEingabe[i].getText (), 8);
                    }
                    else if (dezimalMenueEintrag.isSelected ())
                    {
                        zahl = Integer.parseInt (zellenEingabe[i].getText ());
                    }
                    else if (hexadezimalMenueEintrag.isSelected ())
                    {
                        zahl = Integer.parseInt (zellenEingabe[i].getText (), 16);
                    }

                    if (nurPositivMenueEintrag.isSelected () && zahl < 0)
                    {
                        nachrichtenAusgabe.setText ("Negative Zahlen sind nicht erlaubt!");
                        zellenEingabe[i].requestFocus ();
                    }
                    else
                    {
                        controller.setzeZelle (speicherSeite * ZELLENZAHL + i, zahl);
                        nachrichtenAusgabe.setText (PLATZHALTER);
                    }
                }
                catch (Exception fehler)
                {
                    nachrichtenAusgabe.setText ("Falsches Zahlenformat!");
                    zellenEingabe[i].requestFocus ();
                }
            }
        }
    }

    /**
     * Erlaube dem Benutzer, das Programm laufen zu lassen.
     * 
     * @param erlaubt Laufen ist genau bei true erlaubt.
     */
    public void erlaubeLaufen (boolean erlaubt)
    {
        laufenMenueEintrag.setEnabled (erlaubt);
        schrittweiseMenueEintrag.setEnabled (erlaubt);
    }

    /**
     * Erlaube dem Benutzer, das Programm zu unterbrechen.
     * 
     * @param erlaubt Unterbrechen ist genau bei true erlaubt.
     */
    public void erlaubeUnterbrechen (boolean erlaubt)
    {
        unterbrechenMenueEintrag.setEnabled (erlaubt);
    }

    /**
     * Erlaube dem Benutzer, das Programm fortzusetzen.
     * 
     * @param erlaubt Fortsetzen ist genau bei true erlaubt.
     */
    public void erlaubeFortsetzen (boolean erlaubt)
    {
        fortsetzenMenueEintrag.setEnabled (erlaubt);
    }

    /**
     * Erlaube dem Benutzer, das Programm abzubrechen.
     * 
     * @param erlaubt Abbrechen ist genau bei true erlaubt.
     */
    public void erlaubeAbbrechen (boolean erlaubt)
    {
        abbrechenMenueEintrag.setEnabled (erlaubt);
    }

    /**
     * Erlaube dem Benutzer, das Programm zu editieren.
     * (Falls die View entsprechende Buttons enthaelt,
     * sollten sie aktiviert werden.)
     * 
     * @param erlaubt Editieren ist genau bei true erlaubt.
     */
    public void erlaubeEditieren (boolean erlaubt)
    {
        programmMenue.setEnabled (erlaubt);

        for (int i = 0; i < BEFEHLSZAHL; i++)
        {
            befehlsAuswahl[i].setEnabled (erlaubt);
            argumentEingabe[i].setEnabled (erlaubt);
        }

        for (int i = 0; i < ZELLENZAHL; i++)
        {
            zellenEingabe[i].setEnabled (erlaubt);
        }
    }

    /**
     * Erlaube dem Benutzer, das Programm zu speichern.
     * 
     * @param erlaubt Speichern ist genau bei true erlaubt.
     */
    public void erlaubeSpeichern (boolean erlaubt)
    {
        speichernMenueEintrag.setEnabled (erlaubt);
    }

    /**
     * Lade Programm und Speicherinhalt.
     */
    public void lade ()
    {
        beschrifteProgrammzeilen ();
        beschrifteSpeicherzellen ();
    }

    /**
     * Faerbe den Befehlzaehler und die Nummer des passenden Befehls.
     */
    public void faerbeBefehlsnummer ()
    {
        aktuelleBefehlsNummer = model.gibBefehlsnummer ();
        
        zeigeProgrammzeile (aktuelleBefehlsNummer);
        
        vordergrund        = befehlsNummer[0].getForeground ();
        hintergrundUnten   = zellenEingabe[0].getBackground ();
        hintergrundAuswahl = befehlsAuswahl[0].getBackground ();
        hintergrundOben    = befehlszaehlerEingabe.getBackground ();
        
        befehlsNummer[aktuelleBefehlsNummer % BEFEHLSZAHL].setForeground (Color.magenta);
        befehlszaehlerEingabe.setBackground (Color.magenta);
    }

    /**
     * Entfaerbe den Befehlzaehler und die Nummer des passenden Befehls.
     */
    public void entfaerbeBefehlsnummer ()
    {
        aktuelleBefehlsNummer = model.gibBefehlsnummer ();
        
        zeigeProgrammzeile (aktuelleBefehlsNummer);
        befehlsNummer[aktuelleBefehlsNummer % BEFEHLSZAHL].setForeground (vordergrund);
        befehlszaehlerEingabe.setBackground (hintergrundOben);
    }

    /**
     * Faerbe das Argument und den Akkumulator.
     */
    public void faerbeArgumentUndAkkumulator ()
    {
        aktuelleBefehlsNummer = model.gibBefehlsnummer ();
        
        zeigeProgrammzeile (aktuelleBefehlsNummer);
        
        argumentEingabe[aktuelleBefehlsNummer % BEFEHLSZAHL].setBackground (Color.cyan);
        akkumulatorEingabe.setBackground (Color.cyan);
    }

    /**
     * Entfaerbe das Argument und den Akkumulator.
     */
    public void entfaerbeArgumentUndAkkumulator ()
    {
        aktuelleBefehlsNummer = model.gibBefehlsnummer ();
        
        zeigeProgrammzeile (aktuelleBefehlsNummer);
        
        argumentEingabe[aktuelleBefehlsNummer % BEFEHLSZAHL].setBackground (hintergrundUnten);
        akkumulatorEingabe.setBackground (hintergrundOben);
    }

    /**
     * Aktualisiere den Akkumulator.
     */
    public void aktualisiereAkkumulator ()
    {
        akkumulatorEingabe.setText (alsZeichenkette (model.gibAkkumulatorinhalt ()));
    }

    /**
     * Faerbe den Akkumulator.
     */
    public void faerbeAkkumulator ()
    {
        aktuelleBefehlsNummer = model.gibBefehlsnummer ();
        
        zeigeProgrammzeile (aktuelleBefehlsNummer);
        akkumulatorEingabe.setBackground (befehlsAuswahl[aktuelleBefehlsNummer % BEFEHLSZAHL].getBackground ());
    }

    /**
     * Entfaerbe den Akkumulator.
     */
    public void entfaerbeAkkumulator ()
    {
        akkumulatorEingabe.setBackground (hintergrundOben);
    }

    /**
     * Faerbe das Argument und die passende Speicherzelle.
     */
    public void faerbeArgumentUndZelle ()
    {
        aktuelleBefehlsNummer = model.gibBefehlsnummer ();
        aktuelleZellenNummer = model.gibZellennummer ();
        
        zeigeProgrammzeile (aktuelleBefehlsNummer);
        zeigeSpeicherzelle (aktuelleZellenNummer);
        
        argumentEingabe[aktuelleBefehlsNummer % BEFEHLSZAHL].setBackground (Color.blue);
        zellenNummer[aktuelleZellenNummer % ZELLENZAHL].setForeground (Color.blue);
    }

    /**
     * Entfaerbe das Argument und die passende Speicherzelle.
     */
    public void entfaerbeArgumentUndZelle ()
    {
        aktuelleBefehlsNummer = model.gibBefehlsnummer ();
        aktuelleZellenNummer = model.gibZellennummer ();
        
        zeigeProgrammzeile (aktuelleBefehlsNummer);
        zeigeSpeicherzelle (aktuelleZellenNummer);
        
        argumentEingabe[aktuelleBefehlsNummer % BEFEHLSZAHL].setBackground (hintergrundUnten);
        zellenNummer[aktuelleZellenNummer % ZELLENZAHL].setForeground (vordergrund);
    }

    /**
     * Faerbe den Speicherzelleninhalt und den Akkumulator.
     */
    public void faerbeZellenInhaltUndAkkumulator ()
    {
        aktuelleZellenNummer  = model.gibZellennummer ();
        
        zeigeSpeicherzelle (aktuelleZellenNummer);
        
        akkumulatorEingabe.setBackground (Color.cyan);
        zellenEingabe[aktuelleZellenNummer % ZELLENZAHL].setBackground (Color.cyan);
    }

    /**
     * Aktualisiere den Speicherzelleninhalt.
     */
    public void aktualisiereZelleninhalt ()
    {
        aktuelleZellenNummer  = model.gibZellennummer ();
        
        zeigeSpeicherzelle (aktuelleZellenNummer);
        
        zellenEingabe[aktuelleZellenNummer % ZELLENZAHL].setText (alsZeichenkette (model.gibZelleninhalt ()));
    }

    /**
     * Faerbe den Speicherzelleninhalt.
     */
    public void faerbeZelleninhalt ()
    {
        aktuelleBefehlsNummer = model.gibBefehlsnummer ();
        aktuelleZellenNummer = model.gibZellennummer ();
        
        zeigeProgrammzeile (aktuelleBefehlsNummer);
        zeigeSpeicherzelle (aktuelleZellenNummer);
        
        zellenEingabe[aktuelleZellenNummer % ZELLENZAHL].setBackground (befehlsAuswahl[aktuelleBefehlsNummer % BEFEHLSZAHL].getBackground ());
    }

    /**
     * Entfaerbe den Speicherzelleninhalt.
     */
    public void entfaerbeZelleninhalt ()
    {
        aktuelleZellenNummer  = model.gibZellennummer ();
        
        zeigeSpeicherzelle (aktuelleZellenNummer);
        
        zellenEingabe[aktuelleZellenNummer % ZELLENZAHL].setBackground (hintergrundUnten);
    }

    /**
     * Faerbe den Speicherzelleninhalt und die indirekt
     * adressierte Speicherzelle.
     */
    public void faerbeZellenInhaltUndZelle ()
    {
    }

    /**
     * Entfaerbe den Speicherzelleninhalt und die indirekt
     * adressierte Speicherzelle.
     */
    public void entfaerbeZellenInhaltUndZelle ()
    {
    }

    /**
     * Aktualisiere den Befehlszaehler.
     */
    public void aktualisiereBefehlszaehler ()
    {
        befehlszaehlerEingabe.setText (alsZeichenkette (model.gibBefehlszaehlerinhalt ()));
    }

    /**
     * Faerbe das Argument und den Befehlszaehler.
     */
    public void faerbeArgumentUndBefehlszaehler ()
    {
        aktuelleBefehlsNummer = model.gibBefehlsnummer ();
        
        zeigeProgrammzeile (aktuelleBefehlsNummer);
        
        argumentEingabe[aktuelleBefehlsNummer % BEFEHLSZAHL].setBackground (Color.cyan);
        befehlszaehlerEingabe.setBackground (Color.cyan);
    }

    /**
     * Faerbe den Befehlszaehler.
     */
    public void faerbeBefehlszaehler ()
    {
        aktuelleBefehlsNummer = model.gibBefehlsnummer ();
        
        zeigeProgrammzeile (aktuelleBefehlsNummer);
        
        befehlszaehlerEingabe.setBackground (befehlsAuswahl[aktuelleBefehlsNummer % BEFEHLSZAHL].getBackground ());
    }

    /**
     * Entfaerbe den Befehlszaehler.
     */
    public void entfaerbeBefehlszaehler ()
    {
        befehlszaehlerEingabe.setBackground (hintergrundOben);
    }

    /**
     * Faerbe den Ladebefehl.
     */
    public void faerbeLadebefehl ()
    {
        aktuelleBefehlsNummer = model.gibBefehlsnummer ();
        
        zeigeProgrammzeile (aktuelleBefehlsNummer);
        befehlsAuswahl[aktuelleBefehlsNummer % BEFEHLSZAHL].setBackground (Color.yellow);
    }

    /**
     * Faerbe den Speicherbefehl.
     */
    public void faerbeSpeicherbefehl ()
    {
        aktuelleBefehlsNummer = model.gibBefehlsnummer ();
        
        zeigeProgrammzeile (aktuelleBefehlsNummer);
        befehlsAuswahl[aktuelleBefehlsNummer % BEFEHLSZAHL].setBackground (Color.green);
    }

    /**
     * Faerbe den Rechenbefehl.
     */
    public void faerbeRechenbefehl ()
    {
        aktuelleBefehlsNummer = model.gibBefehlsnummer ();
        
        zeigeProgrammzeile (aktuelleBefehlsNummer);
        befehlsAuswahl[aktuelleBefehlsNummer % BEFEHLSZAHL].setBackground (Color.pink);
    }

    /**
     * Faerbe den Sprungbefehl.
     */
    public void faerbeSprungbefehl ()
    {
        aktuelleBefehlsNummer = model.gibBefehlsnummer ();
        
        zeigeProgrammzeile (aktuelleBefehlsNummer);
        befehlsAuswahl[aktuelleBefehlsNummer % BEFEHLSZAHL].setBackground (Color.orange);
    }

    /**
     * Entfaerbe den Befehl.
     */
    public void entfaerbeBefehl ()
    {
        aktuelleBefehlsNummer = model.gibBefehlsnummer ();
        
        zeigeProgrammzeile (aktuelleBefehlsNummer);
        befehlsAuswahl[aktuelleBefehlsNummer % BEFEHLSZAHL].setBackground (hintergrundAuswahl);
    }
    
    /**
     * Brich ab mit Nachricht.
     * 
     * @param nachricht Die Nachricht, die beim Abbrechen gesandt wird.
     */
    public void brichAbMitNachricht (String nachricht)
    {
        nachrichtenAusgabe.setText (nachricht);
        
        controller.brichAb ();
    }

    /**
     * Beschrifte die Programmzeilen.
     */
    private void beschrifteProgrammzeilen ()
    {
        programmZeile = model.gibProgramm ();
        argument      = model.gibArgumente ();

        int zeile = programmSeite * BEFEHLSZAHL;

        for (int i = 0; i < BEFEHLSZAHL; i++)
        {
            befehlsNummer[i].setText (alsZeichenkette (zeile));

            if (zeile >= programmZeile.size () || programmZeile.get (zeile) == null)
            {
                befehlsAuswahl[i].setSelectedItem (Befehl.values () [0]);
            }
            else
            {
                befehlsAuswahl[i].setSelectedItem (programmZeile.get (zeile));
            }

            if (((Befehl) befehlsAuswahl[i].getSelectedItem ()).brauchtArgument () == 0)
            {
                argumentEingabe[i].setText ("");
                argumentEingabe[i].setEditable (false);
            }
            else
            {  
                argumentEingabe[i].setEditable (true);

                if (zeile >= argument.size () || argument.get (zeile) == null)
                {
                    argumentEingabe[i].setText ("0");
                }
                else
                {
                    argumentEingabe[i].setText (alsZeichenkette (argument.get (zeile)));
                }   
            } 

            zeile++;
        }

        programmZurueckKnopf.setEnabled (programmSeite > 0);
    }

    /**
     * Beschrifte die Speicherzellen.
     */
    private void beschrifteSpeicherzellen ()
    {
        zellenInhalt = model.gibSpeicherbelegung ();

        int zelle = speicherSeite * ZELLENZAHL;

        for (int i = 0; i < ZELLENZAHL; i++)
        {
            zellenNummer[i].setText (alsZeichenkette (zelle));

            if (zelle >= zellenInhalt.size () || zellenInhalt.get (zelle) == null)
            {
                zellenEingabe[i].setText ("0");
            }
            else
            {
                zellenEingabe[i].setText (alsZeichenkette (zellenInhalt.get (zelle)));
            }   

            zelle++;
        } 

        speicherZurueckKnopf.setEnabled (speicherSeite > 0);
    }

    /**
     * Zeige die genannte Programmzeile.
     * 
     * @param zeile Die Nummer der Programmzeile, die gezeigt werden soll.
     */
    private void zeigeProgrammzeile (int zeile)
    {
        programmSeite = zeile / BEFEHLSZAHL;

        beschrifteProgrammzeilen ();
    }

    /**
     * Zeige die genannte Speicherzelle.
     * 
     * @param zelle Die Nummer der Speicherzelle, die gezeigt werden soll.
     */
    private void zeigeSpeicherzelle (int zelle)
    {
        speicherSeite = zelle / ZELLENZAHL;

        beschrifteSpeicherzellen ();
    }

    /**
     * Stelle die Zahl als Zeichenkette dar.
     * 
     * @param zahl Die Zahl, die als Zeichenkette dargestellt werden soll.
     */
    private String alsZeichenkette (int zahl)
    {
        String vorzeichen = "";
        String rueckgabe  = "";

        if (zahl < 0)
        {
            vorzeichen = "-";
        }

        int absoluterWert = Math.abs (zahl);

        if (binaerMenueEintrag.isSelected ())
        {
            rueckgabe = vorzeichen + Integer.toBinaryString (absoluterWert);
        }
        else if (oktalMenueEintrag.isSelected ())
        {
            rueckgabe = vorzeichen + Integer.toOctalString (absoluterWert);
        }
        else if (dezimalMenueEintrag.isSelected ())
        {
            rueckgabe = Integer.toString (zahl);
        }
        else if (hexadezimalMenueEintrag.isSelected ())
        {
            rueckgabe = vorzeichen + Integer.toHexString (absoluterWert);
        }

        return rueckgabe;
    }
}
