import java.io.File;
import java.util.ArrayList;

/**
 * Eine Registermaschine.
 * 
 * @author Bernd Gramlich
 * @version 2015-01-19
 */

public class Maschine implements MaschinenModel
{
    /*
     * Das Programm als Liste von Programmzeilen.
     */
    private ArrayList<Befehl> programmZeile;

    /*
     * Die Argumente der einzelnen Programmzeilen.
     */
    private ArrayList<Integer> argument;

    /*
     * Die Inhalte der einzelnen Speicherzellen.
     */
    private ArrayList<Integer> zellenInhalt;

    /*
     * Der Akkumulator.
     */
    private int akkumulator;

    /*
     * Der Befehlszaehler.
     */
    private int befehlszaehler;

    /*
     * Der aktuelle Befehl (meistens gleich dem Befehlszaehler).
     */
    private int aktuelleBefehlsNummer;

    /*
     * Die aktuelle Speicherzelle.
     */
    private int aktuelleZellenNummer;

    /*
     * Sind nur positive Zahlen und Null erlaubt?
     */
    private boolean nurPositiv;

    /*
     * Laeuft die Maschine?
     */
    private boolean istAktiv;

    /*
     * Laeuft die Maschine schrittweise?
     */
    private boolean laeuftSchrittweise;

    /*
     * Die Liste unserer Beobachter.
     */
    private ArrayList<MaschinenBeobachter> maschinenBeobachter;

    /**
     * Konstruktor.
     * Erzeuge eine Registermaschine.
     */
    public Maschine ()
    {
        /*
         * Beginne mit leerem Programm und leerem Speicherinhalt.
         */
        programmZeile = new ArrayList<Befehl> ();
        argument      = new ArrayList<Integer> ();
        zellenInhalt  = new ArrayList<Integer> ();

        /*
         * Setze alle Zaehler auf Anfang.
         */
        akkumulator           = 0;
        befehlszaehler        = 0;
        aktuelleBefehlsNummer = 0;
        aktuelleZellenNummer  = 0;

        /*
         * Rechne mit allen ganzen Zahlen.
         */
        nurPositiv = false;

        /*
         * Beginne mit einer leeren Liste von Beobachtern.
         */
        maschinenBeobachter = new ArrayList<MaschinenBeobachter> ();
    }

    /**
     * Beginne ein neues Programm.
     */
    public void neu ()
    {
        programmZeile = new ArrayList<Befehl> ();
        argument      = new ArrayList<Integer> ();
        zellenInhalt  = new ArrayList<Integer> ();
        
        benachrichtigeBeobachter ();
    }

    /**
     * Lade das Programm aus der Programmdatei.
     * 
     * @param programmDatei Die Datei, aus der das Programm geladen werden soll.
     */
    public void ladeProgramm (File programmDatei)
    {
        /*
         * Das Laden uebernimmt die Klasse Datenleser.
         */
        //labyrinth = Datenleser.gibDatenleser ().lies (labyrinthDatei);

        benachrichtigeBeobachter ();
    }

    /**
     * Speichere das Programm in die Programmdatei.
     * 
     * @param programmDatei Die Datei, in die das Programm gespeichert werden soll.
     */
    public boolean speichereProgramm (File programmDatei)
    {
        //return Datenschreiber.gibDatenschreiber ().schreibe (labyrinth, labyrinthDatei, positionX, positionY, blickrichtung);
        return true;
    }

    /**
     * Lass das Programm laufen.
     */
    public void laufe ()
    {
        istAktiv           = true;
        laeuftSchrittweise = false;

        (new LaufeLos ()).start ();
    }

    /**
     * Lass das Programm schrittweise laufen.
     */
    public void laufeSchrittweise ()
    {
        istAktiv           = true;
        laeuftSchrittweise = true;

        (new LaufeLos ()).start ();
    }

    /**
     * Unterbrich das Programm.
     */
    public void unterbrich ()
    {
        istAktiv = false;
    }

    /**
     * Setze das Programm fort.
     */
    public void setzeFort ()
    {
        (new LaufeLos ()).start ();
    }

    /**
     * Brich das Programm ab.
     */
    public void brichAb ()
    {
    }

    /**
     * Schreibe einen Befehl in eine Programmzeile.
     * 
     * @param zeile Die Programmzeile, in die der Befehl geschrieben werden soll.
     * @param kommando Der Befehl, der in die Programmzeile geschrieben werden soll.
     */
    public void setzeBefehl (int zeile, Befehl kommando)
    {
        int diff = zeile - programmZeile.size ();

        while (diff >= 0)
        {
            programmZeile.add (null);
            diff--;
        }

        programmZeile.set (zeile, kommando);
    }

    /**
     * Schreibe ein Argument in eine Programmzeile.
     * 
     * @param zeile Die Programmzeile, in die das Argument geschrieben werden soll.
     * @param zahl Das Argument, das in die Programmzeile geschrieben werden soll.
     * 
     * @return True genau dann, wenn das Argument korrekt war.
     */
    public boolean setzeArgument (int zeile, int zahl)
    {
        int diff = zeile - argument.size ();

        while (diff >= 0)
        {
            argument.add (null);
            diff--;
        }

        argument.set (zeile, zahl);

        return true;
    }

    /**
     * Schreibe eine Zahl in eine Speicherzelle.
     * 
     * @param zelle Die Speicherzelle, in die die Zahl geschrieben werden soll.
     * @param zahl Die Zahl, die in die Speicherzelle geschrieben werden soll.
     * 
     * @return True genau dann, wenn die Zahl korrekt war.
     */
    public boolean setzeZelle (int zelle, int zahl)
    {
        int diff = zelle - zellenInhalt.size ();

        while (diff >= 0)
        {
            zellenInhalt.add (0);
            diff--;
        }

        zellenInhalt.set (zelle, zahl);

        return true;
    }

    /**
     * Nenne die Programmbefehle.
     * 
     * @return Die Liste der Programmbefehle.
     */
    public ArrayList<Befehl> gibProgramm ()
    {
        return programmZeile;
    }

    /**
     * Nenne die Argumente der Programmbefehle.
     * 
     * @return Die Liste der Argumente der Programmbefehle.
     */
    public ArrayList<Integer> gibArgumente ()
    {
        return argument;
    }

    /**
     * Nenne die Speicherbelegung.
     * 
     * @return Die Liste der Zelleninhalte.
     */
    public ArrayList<Integer> gibSpeicherbelegung ()
    {
        return zellenInhalt;
    }

    /**
     * Nenne die Nummer des aktuellen Befehls.
     * 
     * @return Die Nummer des aktuellen Befehls.
     */
    public int gibBefehlsnummer ()
    {
        return aktuelleBefehlsNummer;
    }

    /**
     * Nenne die Nummer der aktuellen Speicherzelle.
     * 
     * @return Die Nummer der aktuellen Speicherzelle.
     */
    public int gibZellennummer ()
    {
        return aktuelleZellenNummer;
    }

    /**
     * Nenne den Inhalt der aktuellen Speicherzelle.
     * 
     * @return Der Inhalt der aktuellen Speicherzelle.
     */
    public int gibZelleninhalt ()
    {
        return zellenInhalt.get (aktuelleZellenNummer);
    }

    /**
     * Nenne den Inhalt des Akkumulators.
     * 
     * @return Der Inhalt des Akkumulators.
     */
    public int gibAkkumulatorinhalt ()
    {
        return akkumulator;
    }

    /**
     * Nenne den Inhalt des Befehlszaehlers.
     * 
     * @return Der Inhalt des Befehlszaehlers.
     */
    public int gibBefehlszaehlerinhalt ()
    {
        return befehlszaehler;
    }

    /**
     * Speichere einen Maschinenbeobachter.
     * 
     * @param b Der Maschinenbeobachter.
     */
    public void speichereAlsBeobachter (MaschinenBeobachter b)
    {
        maschinenBeobachter.add (b);
    }

    /**
     * Loesche einen Maschinenbeobachter.
     * 
     * @param b Der Maschinenbeobachter.
     */
    public void loescheAlsBeobachter (MaschinenBeobachter b)
    {
        maschinenBeobachter.remove (b);
    }

    /**
     * Benachrichtige Maschinenbeobachter, dass sich an der Maschine
     * etwas geaendert hat.
     */
    private void benachrichtigeBeobachter ()
    {
        for (MaschinenBeobachter b: maschinenBeobachter)
        {
            b.lade ();
        }
    }

    /**
     * Lasse die Maschine loslaufen.
     */
    private class LaufeLos extends Thread
    {
        /**
         * Ablaufplan.
         */
        public void run ()
        {
            if (istAktiv)
            {
                akkumulator    = 0;
                befehlszaehler = 0;
            }
            else
            {
                istAktiv = true;
            }

            for (MaschinenBeobachter b: maschinenBeobachter)
            {
                b.aktualisiereAkkumulator ();
                b.aktualisiereBefehlszaehler ();
            }

            while (istAktiv)
            {
                aktuelleBefehlsNummer = befehlszaehler;

                if (laeuftSchrittweise)
                {
                    for (MaschinenBeobachter b: maschinenBeobachter)
                    {
                        b.faerbeBefehlsnummer ();
                    }

                    pausiere ();
                }

                Befehl kommando = Befehl.values () [0];

                if (programmZeile.size () > aktuelleBefehlsNummer)
                {
                    kommando = programmZeile.get (aktuelleBefehlsNummer);
                }

                Integer zahl = null;

                if (argument.size () > aktuelleBefehlsNummer)
                {
                    zahl = argument.get (aktuelleBefehlsNummer);
                }

                int wert = 0;

                switch (kommando)
                {
                    case LOAD:
                    aktuelleZellenNummer = zahl;

                    if (laeuftSchrittweise)
                    {
                        for (MaschinenBeobachter b: maschinenBeobachter)
                        {
                            b.faerbeLadebefehl ();
                        }

                        pausiere ();

                        for (MaschinenBeobachter b: maschinenBeobachter)
                        {
                            b.faerbeArgumentUndZelle ();
                        }

                        pausiere ();

                        for (MaschinenBeobachter b: maschinenBeobachter)
                        {
                            b.faerbeZellenInhaltUndAkkumulator ();
                        }

                        pausiere ();
                    }

                    if (zellenInhalt.size () > zahl)
                    {
                        wert = zellenInhalt.get (zahl);
                    }

                    akkumulator = wert;

                    for (MaschinenBeobachter b: maschinenBeobachter)
                    {
                        b.aktualisiereAkkumulator ();
                    }

                    if (laeuftSchrittweise)
                    {
                        for (MaschinenBeobachter b: maschinenBeobachter)
                        {
                            b.faerbeAkkumulator ();
                        }

                        pausiere ();
                    }

                    befehlszaehler++;

                    for (MaschinenBeobachter b: maschinenBeobachter)
                    {
                        b.aktualisiereBefehlszaehler ();
                    }

                    break;

                    case STORE:
                    aktuelleZellenNummer = zahl;

                    if (laeuftSchrittweise)
                    {
                        for (MaschinenBeobachter b: maschinenBeobachter)
                        {
                            b.faerbeSpeicherbefehl ();
                        }

                        pausiere ();

                        for (MaschinenBeobachter b: maschinenBeobachter)
                        {
                            b.faerbeArgumentUndZelle ();
                        }

                        pausiere ();

                        for (MaschinenBeobachter b: maschinenBeobachter)
                        {
                            b.faerbeZellenInhaltUndAkkumulator ();
                        }

                        pausiere ();
                    }

                    setzeZelle (zahl, akkumulator);

                    for (MaschinenBeobachter b: maschinenBeobachter)
                    {
                        b.aktualisiereZelleninhalt ();
                    }

                    if (laeuftSchrittweise)
                    {
                        for (MaschinenBeobachter b: maschinenBeobachter)
                        {
                            b.faerbeZelleninhalt ();
                        }

                        pausiere ();
                    }

                    befehlszaehler++;

                    for (MaschinenBeobachter b: maschinenBeobachter)
                    {
                        b.aktualisiereBefehlszaehler ();
                    }

                    break;

                    case CLOAD:

                    if (laeuftSchrittweise)
                    {
                        for (MaschinenBeobachter b: maschinenBeobachter)
                        {
                            b.faerbeLadebefehl ();
                        }

                        pausiere ();

                        for (MaschinenBeobachter b: maschinenBeobachter)
                        {
                            b.faerbeArgumentUndAkkumulator ();
                        }

                        pausiere ();
                    }

                    akkumulator = zahl;

                    for (MaschinenBeobachter b: maschinenBeobachter)
                    {
                        b.aktualisiereAkkumulator ();
                    }

                    if (laeuftSchrittweise)
                    {
                        for (MaschinenBeobachter b: maschinenBeobachter)
                        {
                            b.faerbeAkkumulator ();
                        }

                        pausiere ();
                    }

                    befehlszaehler++;

                    for (MaschinenBeobachter b: maschinenBeobachter)
                    {
                        b.aktualisiereBefehlszaehler ();
                    }

                    break;

                    case ADD:
                    case SUB:
                    case MULT:
                    case DIV:
                    aktuelleZellenNummer = zahl;

                    if (laeuftSchrittweise)
                    {
                        for (MaschinenBeobachter b: maschinenBeobachter)
                        {
                            b.faerbeRechenbefehl ();
                        }

                        pausiere ();

                        for (MaschinenBeobachter b: maschinenBeobachter)
                        {
                            b.faerbeArgumentUndZelle ();
                        }

                        pausiere ();

                        for (MaschinenBeobachter b: maschinenBeobachter)
                        {
                            b.faerbeZellenInhaltUndAkkumulator ();
                        }

                        pausiere ();
                    }

                    if (zellenInhalt.size () > zahl)
                    {
                        wert = zellenInhalt.get (zahl);
                    }

                    switch (kommando)
                    {
                        case ADD:
                        akkumulator += wert;
                        break;

                        case SUB:
                        akkumulator -= wert;

                        if (nurPositiv && akkumulator < 0)
                        {
                            akkumulator = 0;
                        }   
                        break;

                        case MULT:
                        akkumulator *= wert;
                        break;

                        case DIV:
                        if (wert == 0)
                        {
                            istAktiv = false;
                            
                            for (MaschinenBeobachter b: maschinenBeobachter)
                            {
                                b.brichAbMitNachricht ("Division durch 0.");
                            }
                        }
                        else
                        {
                            akkumulator = akkumulator / wert;
                        }
                        break;
                    }

                    for (MaschinenBeobachter b: maschinenBeobachter)
                    {
                        b.aktualisiereAkkumulator ();
                    }

                    if (laeuftSchrittweise)
                    {
                        for (MaschinenBeobachter b: maschinenBeobachter)
                        {
                            b.faerbeAkkumulator ();
                        }

                        pausiere ();
                    }

                    befehlszaehler++;

                    for (MaschinenBeobachter b: maschinenBeobachter)
                    {
                        b.aktualisiereBefehlszaehler ();
                    }

                    break;

                    case JUMP:
                    case JGT:
                    case JGE:
                    case JEQ:
                    case JLE:
                    case JLT:
                    aktuelleZellenNummer = zahl;

                    if (laeuftSchrittweise)
                    {
                        for (MaschinenBeobachter b: maschinenBeobachter)
                        {
                            b.faerbeSprungbefehl ();
                        }

                        pausiere ();

                        for (MaschinenBeobachter b: maschinenBeobachter)
                        {
                            b.faerbeArgumentUndBefehlszaehler ();
                        }

                        pausiere ();
                    }

                    befehlszaehler++;

                    switch (kommando)
                    {
                        case JUMP:
                        befehlszaehler = zahl;
                        break;

                        case JGT:
                        if (akkumulator > 0)
                        { 
                            befehlszaehler = zahl;
                        }
                        break;

                        case JGE:
                        if (akkumulator >= 0)
                        { 
                            befehlszaehler = zahl;
                        }
                        break;

                        case JEQ:
                        if (akkumulator == 0)
                        { 
                            befehlszaehler = zahl;
                        }
                        break;

                        case JLE:
                        if (akkumulator <= 0)
                        { 
                            befehlszaehler = zahl;
                        }
                        break;

                        case JLT:
                        if (akkumulator < 0)
                        { 
                            befehlszaehler = zahl;
                        }
                        break;
                    }

                    for (MaschinenBeobachter b: maschinenBeobachter)
                    {
                        b.aktualisiereBefehlszaehler ();
                    }

                    if (laeuftSchrittweise)
                    {
                        for (MaschinenBeobachter b: maschinenBeobachter)
                        {
                            b.faerbeBefehlszaehler ();
                        }

                        pausiere ();
                    }

                    break;

                    case INC:
                    case DEC:
                    if (laeuftSchrittweise)
                    {
                        for (MaschinenBeobachter b: maschinenBeobachter)
                        {
                            b.faerbeRechenbefehl ();
                        }

                        pausiere ();
                    }

                    switch (kommando)
                    {
                        case INC:
                        akkumulator ++;
                        break;

                        case DEC:
                        akkumulator --;

                        if (nurPositiv && akkumulator < 0)
                        {
                            akkumulator = 0;
                        }   
                        break;
                    }

                    for (MaschinenBeobachter b: maschinenBeobachter)
                    {
                        b.aktualisiereAkkumulator ();
                    }

                    if (laeuftSchrittweise)
                    {
                        for (MaschinenBeobachter b: maschinenBeobachter)
                        {
                            b.faerbeAkkumulator ();
                        }

                        pausiere ();
                    }

                    befehlszaehler++;

                    for (MaschinenBeobachter b: maschinenBeobachter)
                    {
                        b.aktualisiereBefehlszaehler ();
                    }

                    break;

                    case END:
                    if (laeuftSchrittweise)
                    {
                        pausiere ();
                    }
                    istAktiv = false;

                    for (MaschinenBeobachter b: maschinenBeobachter)
                    {
                        b.brichAbMitNachricht ("Programmende.");
                    }
                    break;

                    default:
                    istAktiv = false;

                    for (MaschinenBeobachter b: maschinenBeobachter)
                    {
                        b.brichAbMitNachricht ("Ungueltiger Befehl.");
                    }
                    break;
                }

                if (laeuftSchrittweise)
                {
                    pausiere ();

                    for (MaschinenBeobachter b: maschinenBeobachter)
                    {
                        b.entfaerbeBefehlsnummer ();
                        b.entfaerbeBefehl ();
                        b.entfaerbeArgumentUndZelle ();
                        b.entfaerbeZelleninhalt ();
                        b.entfaerbeAkkumulator ();
                        b.entfaerbeBefehlszaehler ();
                    }

                    pausiere ();
                }
            }
        }

        /**
         * Pausiere.
         */
        private void pausiere ()
        {
            try
            {
                Thread.sleep (1000);
            }
            catch (Exception fehler)
            {
            }
        }
    }
}