/**
 * Ein Beobachter eines Registermaschine, der immer den aktuellen
 * Stand des editierten oder laufenden Programms erfahren soll.
 * 
 * @author Bernd Gramlich
 * @version 2015-01-19
 */
public interface MaschinenBeobachter
{
    /**
     * Lade Programm und Speicherinhalt.
     */
    void lade ();
    
    /**
     * Faerbe den Befehlzaehler und die Nummer des passenden Befehls.
     */
    void faerbeBefehlsnummer ();
    
    /**
     * Entfaerbe den Befehlzaehler und die Nummer des passenden Befehls.
     */
    void entfaerbeBefehlsnummer ();
    
    /**
     * Faerbe das Argument und den Akkumulator.
     */
    void faerbeArgumentUndAkkumulator ();
    
    /**
     * Entfaerbe das Argument und den Akkumulator.
     */
    void entfaerbeArgumentUndAkkumulator ();
    
    /**
     * Aktualisiere den Akkumulator.
     */
    void aktualisiereAkkumulator ();
    
    /**
     * Faerbe den Akkumulator.
     */
    void faerbeAkkumulator ();
    
    /**
     * Entfaerbe den Akkumulator.
     */
    void entfaerbeAkkumulator ();
    
    /**
     * Faerbe das Argument und die passende Speicherzelle.
     */
    void faerbeArgumentUndZelle ();
    
    /**
     * Entfaerbe das Argument und die passende Speicherzelle.
     */
    void entfaerbeArgumentUndZelle ();
    
    /**
     * Faerbe den Speicherzelleninhalt und den Akkumulator.
     */
    void faerbeZellenInhaltUndAkkumulator ();
    
    /**
     * Aktualisiere den Speicherzelleninhalt.
     */
    void aktualisiereZelleninhalt ();
    
    /**
     * Faerbe den Speicherzelleninhalt.
     */
    void faerbeZelleninhalt ();
    
    /**
     * Entfaerbe den Speicherzelleninhalt.
     */
    void entfaerbeZelleninhalt ();
    
    /**
     * Faerbe den Speicherzelleninhalt und die indirekt
     * adressierte Speicherzelle.
     */
    void faerbeZellenInhaltUndZelle ();
    
    /**
     * Entfaerbe den Speicherzelleninhalt und die indirekt
     * adressierte Speicherzelle.
     */
    void entfaerbeZellenInhaltUndZelle ();
    
    /**
     * Aktualisiere den Befehlszaehler.
     */
    void aktualisiereBefehlszaehler ();
    
    /**
     * Faerbe das Argument und den Befehlszaehler.
     */
    void faerbeArgumentUndBefehlszaehler ();
    
    /**
     * Faerbe den Befehlszaehler.
     */
    void faerbeBefehlszaehler ();
    
    /**
     * Entfaerbe den Befehlszaehler.
     */
    void entfaerbeBefehlszaehler ();
    
    /**
     * Faerbe den Ladebefehl.
     */
    void faerbeLadebefehl ();
    
    /**
     * Faerbe den Speicherbefehl.
     */
    void faerbeSpeicherbefehl ();
    
    /**
     * Faerbe den Rechenbefehl.
     */
    void faerbeRechenbefehl ();
    
    /**
     * Faerbe den Sprungbefehl.
     */
    void faerbeSprungbefehl ();
    
    /**
     * Entfaerbe den Befehl.
     */
    void entfaerbeBefehl ();
    
    /**
     * Brich ab mit Nachricht.
     * 
     * @param nachricht Die Nachricht, die beim Abbrechen gesandt wird.
     */
    void brichAbMitNachricht (String nachricht);
}