import java.io.File;

/**
 * Die Steuerung der Registermaschine.
 * 
 * @author Bernd Gramlich
 * @version 2015-01-19
 */
public class MaschinenSteuerung implements MaschinenController
{
    /*
     * Das Maschinenmodell, das ich verwalte.
     */
    private MaschinenModel model;

    /*
     * Die Maschinenpraesentation, die ich verwalte.
     */
    private MaschinenView view;

    /*
     * Mein interner Zustand. Wenn nur positive Zahlen
     * erlaubt sind, muessen gewisse Eingaben zurueckgewiesen
     * werden.
     */
    private boolean positiv;

    /**
     * Konstruktor.
     * Ordne mir ein MaschinenModel m unter.
     */
    public MaschinenSteuerung (MaschinenModel m)
    {
        model = m;

        positiv = false;
    }

    /**
     * Ordne mir eine neue Praesentation (View) unter.
     */
    public void setzeView (MaschinenView v)
    {
        view = v;

        view.erlaubeSpeichern (false);
        view.erlaubeLaufen (true);
        view.erlaubeUnterbrechen (false);
        view.erlaubeFortsetzen (false);
        view.erlaubeAbbrechen (false);
    }

    /**
     * Beginne ein neues Programm.
     */
    public void neu ()
    {
        model.neu ();
    }

    /**
     * Lade das Programm aus der Programmdatei.
     * 
     * @param programmDatei Die Datei, aus der das Programm geladen werden soll.
     */
    public void ladeProgramm (File programmDatei)
    {
    }

    /**
     * Speichere das Programm in die Programmdatei.
     * 
     * @param programmDatei Die Datei, in die das Programm gespeichert werden soll.
     */
    public void speichereProgramm (File programmDatei)
    {
    }

    /**
     * Beende die Maschine.
     */
    public void ende ()
    {
        System.exit (0);
    }

    /**
     * Lass das Programm laufen.
     */
    public void laufe ()
    {
        view.erlaubeLaufen (false);
        view.erlaubeUnterbrechen (true);
        view.erlaubeAbbrechen (true);
        view.erlaubeEditieren (false);
        
        model.laufe ();
    }

    /**
     * Lass das Programm schrittweise laufen.
     */
    public void laufeSchrittweise ()
    {
        view.erlaubeLaufen (false);
        view.erlaubeUnterbrechen (true);
        view.erlaubeAbbrechen (true);
        view.erlaubeEditieren (false);
        
        model.laufeSchrittweise ();
    }

    /**
     * Unterbrich das Programm.
     */
    public void unterbrich ()
    {
       view.erlaubeUnterbrechen (false);
       view.erlaubeFortsetzen (true);
       
       model.unterbrich ();
    }
    
    /**
     * Setze das Programm fort.
     */
    public void setzeFort ()
    {
       view.erlaubeUnterbrechen (true);
       view.erlaubeFortsetzen (false);
       
       model.setzeFort ();
    }

    /**
     * Brich das Programm ab.
     */
    public void brichAb ()
    {
       view.erlaubeLaufen (true);
       view.erlaubeUnterbrechen (false);
       view.erlaubeFortsetzen (false);
       view.erlaubeAbbrechen (false);
       view.erlaubeEditieren (true);
       
       model.brichAb ();
    }

    /**
     * Schreibe einen Befehl in eine Programmzeile.
     * 
     * @param zeile Die Programmzeile, in die der Befehl geschrieben werden soll.
     * @param kommando Der Befehl, der in die Programmzeile geschrieben werden soll.
     */
    public void setzeBefehl (int zeile, Befehl kommando)
    {
        model.setzeBefehl (zeile, kommando);
    }

    /**
     * Schreibe ein Argument in eine Programmzeile.
     * 
     * @param zeile Die Programmzeile, in die das Argument geschrieben werden soll.
     * @param zahl Das Argument, das in die Programmzeile geschrieben werden soll.
     * 
     * @return True genau dann, wenn das Argument korrekt war.
     */
    public boolean setzeArgument (int zeile, int zahl)
    {
        return model.setzeArgument (zeile, zahl);
    }

    /**
     * Schreibe eine Zahl in eine Speicherzelle.
     * 
     * @param zelle Die Speicherzelle, in die die Zahl geschrieben werden soll.
     * @param zahl Die Zahl, die in die Speicherzelle geschrieben werden soll.
     * 
     * @return True genau dann, wenn die Zahl korrekt war.
     */
    public boolean setzeZelle (int zelle, int zahl)
    {
        return model.setzeZelle (zelle, zahl);
    }
}